<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * File Storage Extension: VideoField
 */
class _VideoField
{
    /**
     * Count stored files
     *
     * @return	int
     */
    public function count()
    {
        $count = 0;

        foreach( \IPS\Db::i()->select( '*', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) ) AS $field )
        {
            $count += \IPS\Db::i()->select( 'COUNT(*)', 'videos_custom_values', array( "field_{$field['cf_id']}<>? OR field_{$field['cf_id']} IS NOT NULL", '' ) )->first();
        }

        return $count;
    }

    /**
     * Move stored files
     *
     * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
     * @param	int			$storageConfiguration	New storage configuration ID
     * @param	int|NULL	$oldConfiguration		Old storage configuration ID
     * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
     * @return	void								FALSE when there are no more files to move
     */
    public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
    {
        if( !\IPS\Db::i()->select( 'COUNT(*)', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) )->first() )
        {
            throw new \Underflowexception;
        }

        foreach( \IPS\Db::i()->select( '*', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) ) AS $field )
        {
            $cfield	= \IPS\Db::i()->select( '*', 'videos_custom_values', array( "field_{$field['cf_id']}<>? OR field_{$field['cf_id']} IS NOT NULL", '' ), 'video_id', array( $offset, 1 ) )->first();

            try
            {
                $file = \IPS\File::get( $oldConfiguration ?: 'videos_VideoField', $cfield[ 'field_' . $field['cf_id'] ] )->move( $storageConfiguration );

                if ( (string) $file != $cfield[ 'field_' . $field['cf_id'] ] )
                {
                    \IPS\Db::i()->update( 'videos_custom_values', array( "field_{$field['cf_id']}=?", (string) $file ), array( 'video_id=?', $cfield['video_id'] ) );
                }
            }
            catch( \Exception $e )
            {
                /* Any issues are logged */
            }
        }
    }

    /**
     * Fix all URLs
     *
     * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
     * @return void
     */
    public function fixUrls( $offset )
    {
        if( !\IPS\Db::i()->select( 'COUNT(*)', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) )->first() )
        {
            throw new \Underflowexception;
        }

        foreach( \IPS\Db::i()->select( '*', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) ) AS $field )
        {
            $cfield	= \IPS\Db::i()->select( '*', 'videos_custom_values', array( "field_{$field['cf_id']}<>? OR field_{$field['cf_id']} IS NOT NULL", '' ), 'video_id', array( $offset, 1 ) )->first();

            if ( $new = \IPS\File::repairUrl( $cfield[ 'field_' . $field['cf_id'] ] ) )
            {
                \IPS\Db::i()->update( 'videos_custom_values', array( "field_{$field['cf_id']}=?", $new ), array( 'video_id=?', $cfield['video_id'] ) );
            }
        }
    }

    /**
     * Check if a file is valid
     *
     * @param	string	$file		The file path to check
     * @return	bool
     */
    public function isValidFile( $file )
    {
        $valid = FALSE;
        foreach( \IPS\Db::i()->select( '*', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) ) AS $field )
        {
            try
            {
                \IPS\Db::i()->select( '*', 'videos_custom_values', array( "field_{$field['cf_id']}=?", (string) $file ) )->first();

                $valid = TRUE;
                break;
            }
            catch( \UnderflowException $e ) {}
        }

        return $valid;
    }

    /**
     * Delete all stored files
     *
     * @return	void
     */
    public function delete()
    {
        foreach( \IPS\Db::i()->select( '*', 'videos_custom_fields', array( 'cf_type=?', 'Upload' ) ) AS $field )
        {
            try
            {
                foreach( \IPS\Db::i()->select( '*', 'videos_custom_values', array( "field_{$field['cf_id']}<>? OR field_{$field['cf_id']} IS NOT NULL", '' ) ) as $cfield )
                {
                    \IPS\File::get( 'videos_VideoField', $cfield[ 'field_' . $field['cf_id'] ] )->delete();
                }
            }
            catch( \Exception $e ){}
        }
    }
}