<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\extensions\core\MemberFilter;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Member Filter Extension
 */
class _Videos
{
    /**
     * Determine if the filter is available in a given area
     *
     * @param	string	$area	Area to check
     * @return	bool
     */
    public function availableIn( $area )
    {
        return \in_array( $area, array( 'bulkmail', 'group_promotions' ) );
    }

    /**
     * Get Setting Field
     *
     * @param	mixed	$criteria	Value returned from the save() method
     * @return	array 	Array of form elements
     */
    public function getSettingField( $criteria )
    {
        $options = array( 'any' => 'any', 'yes' => 'yes', 'no' => 'no' );

        return array(
            new \IPS\Helpers\Form\Radio( 'mf_has_video', isset( $criteria['has_video'] ) ? $criteria['has_video'] : 'any', FALSE, array( 'options' => $options ) ),
        );
    }

    /**
     * Save the filter data
     *
     * @param	array	$post	Form values
     * @return	mixed			False, or an array of data to use later when filtering the members
     * @throws \LogicException
     */
    public function save( $post )
    {
        if( isset( $post['mf_has_video'] ) and \in_array( $post['mf_has_video'], array( 'yes', 'no' ) ) )
        {
            return array( 'has_video' => $post['mf_has_video'] );
        }

        return FALSE;
    }

    /**
     * Get where clause to add to the member retrieval database query
     *
     * @param	mixed				$data	The array returned from the save() method
     * @return	string|array|NULL	Where clause
     */
    public function getQueryWhereClause( $data )
    {
        if( isset( $data['has_video'] ) )
        {
            $videoAuthorsSelect = \IPS\Db::i()->select( 'video_author_id', 'videos_videos', NULL, NULL, NULL, NULL, NULL, \IPS\Db::SELECT_DISTINCT );
            return ( $data['has_video'] == 'yes' ) ? array( array( 'core_members.member_id IN(' . $videoAuthorsSelect . ')' ) ) : array( array( 'core_members.member_id NOT IN(' . $videoAuthorsSelect . ')' ) );
        }

        return NULL;
    }

    /**
     * Determine if a member matches specified filters
     *
     * @note	This is only necessary if availableIn() includes group_promotions
     * @param	\IPS\Member	$member		Member object to check
     * @param	array 		$filters	Previously defined filters
     * @return	bool
     */
    public function matches( \IPS\Member $member, $filters )
    {
        /* If we aren't filtering by this, then any member matches */
        if( !isset( $filters['has_video'] ) OR !$filters['has_video'] )
        {
            return TRUE;
        }

        $result = (bool) \IPS\Db::i()->select( 'COUNT(*)', 'videos_videos', array( 'video_author_id=?', $member->member_id ) )->first();

        return ( $filters['has_video'] == 'no' ) ? !$result : $result;
    }
}