<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\modules\admin\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Video Custom Fields
 */
class _videofields extends \IPS\Node\Controller
{
    /**
     * @brief	Has been CSRF-protected
     */
    public static $csrfProtected = TRUE;

    /**
     * Node Class
     */
    protected $nodeClass = '\IPS\videos\Field';

    /**
     * Execute
     *
     * @return	void
     */
    public function execute()
    {
        \IPS\Dispatcher::i()->checkAcpPermission( 'videofields_manage' );
        parent::execute();
    }

    /**
     * Get Root Buttons
     *
     * @return	array
     */
    public function _getRootButtons()
    {
        $buttons = parent::_getRootButtons();

        /* Change form button link */
        if ( isset( $buttons['add'] ) )
        {
            $buttons['add']['link'] = $buttons['add']['link']->setQueryString(array('category' => (int)\IPS\Request::i()->category));
        }

        return $buttons;
    }

    /**
     * Get Root Rows
     *
     * @return	array
     */
    public function _getRoots()
    {
        $where = array();

        /* Filter by category? */
        if( \IPS\Request::i()->category )
        {
            $where = array( 'cf_category_id=?', (int) \IPS\Request::i()->category );
        }

        /* Get custom fields like before. */
        $nodeClass = $this->nodeClass;
        $rows = array();
        foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'videos_custom_fields', $where, 'cf_position ASC', 50 ), $nodeClass ) as $node )
        {
            $rows[ $node->id ] = $this->_getRow( $node );
        }
        return $rows;
    }

    /**
     * Redirect after save
     *
     * @param	\IPS\Node\Model	$old			A clone of the node as it was before or NULL if this is a creation
     * @param	\IPS\Node\Model	$new			The node now
     * @param	string			$lastUsedTab	The tab last used in the form
     * @return	void
     */
    protected function _afterSave( \IPS\Node\Model $old = NULL, \IPS\Node\Model $new, $lastUsedTab = FALSE )
    {
        if( \IPS\Request::i()->isAjax() )
        {
            \IPS\Output::i()->json( array() );
        }
        else
        {
            /* Redirect back to our category */
            \IPS\Output::i()->redirect( $this->url->setQueryString( array( 'category' => $new->category_id ) ), 'saved' );
        }
    }
}