<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\modules\admin\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * videoimport
 */
class _videoimport extends \IPS\Node\Controller
{
    /**
     * @brief	Has been CSRF-protected
     */
    public static $csrfProtected = TRUE;

	/**
	 * Node Class
	 */
	protected $nodeClass = '\IPS\videos\Import';
	
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'videoimport_manage' );
		parent::execute();
	}

    /**
     * Run import
     *
     * @return	void
     */
    public function run()
    {
        \IPS\Dispatcher::i()->checkAcpPermission( 'videoimport_run' );

        try
        {
            $feed = \IPS\videos\Import::load( \IPS\Request::i()->id );
            $feed->run();
        }
        catch ( \OutOfRangeException $e )
        {
            \IPS\Output::i()->error( 'node_error', '', 404, '' );
        }
        catch ( \Exception $e )
        {
            \IPS\Output::i()->error( 'rss_run_error', '', 500, '' );
        }

        \IPS\Session::i()->log( 'acplog__videoimport_ran', array( $feed->title => FALSE ) );
        \IPS\Output::i()->redirect( \IPS\Http\Url::internal('app=videos&module=videos&controller=videoimport' ) );
    }
}