<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\modules\front\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * browse
 */
class _browse extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
        
        /* We need at least 1 category */
        if( !\count( \IPS\videos\Category::roots() ) )
        {      
            \IPS\Output::i()->error( 'no_video_categories_found', '', 404, '' );
        }
        
		/* Latest videos RSS feed link */
		if( \IPS\Settings::i()->vs_enable_rss  )
		{
			\IPS\Output::i()->rssFeeds[ 'latest_videos_main' ] = \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&do=rss', 'front', 'videos_rss' );
		}  
	}

	/**
	 * View Category
	 *
	 * @return	void
	 */
	protected function manage()
	{
	    /* Get category */
		try
		{
            $category = \IPS\videos\Category::loadAndCheckPerms( \IPS\Request::i()->id, 'view' );
            
            /* Decode options */        
            $categoryOptions = json_decode( $category->options );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '', 404, '' );
		}

        /* Set list type cookie */
        if( \IPS\Request::i()->listType && \in_array( \IPS\Request::i()->listType, array( 'list', 'thumb' ) ) )
        {
            \IPS\Request::i()->cookie['listType'] = \IPS\Request::i()->listType;
            \IPS\Request::i()->setCookie( 'listType', \IPS\Request::i()->listType, NULL, FALSE );          
        }

        /* Default category to list */
        if( ( isset( $categoryOptions->display_type ) AND $categoryOptions->display_type ) AND !\IPS\Request::i()->listType )
        {
            \IPS\Request::i()->cookie['listType'] = 'list';    
        }
        
        /* Pull videos from viewing category */
        $catIDS[ $category->id ] = $category->id;
        
        /* Add any children cats */
        if( ( isset( $categoryOptions->sub_videos ) AND $categoryOptions->sub_videos ) AND \count( $category->children() ) )
        {
            foreach( $category->children() as $child ) 
            {
                $catIDS[ $child->id ] = $child->id;    

				if ( \count( $child->children() ) )
				{
					foreach ( $child->children() AS $_child )
					{
						$catIDS[ $_child->id ] = $_child->id;
					}
				} 
            }     
        } 

        /* Get category videos */
		$table = new \IPS\Helpers\Table\Content( '\IPS\videos\Video', $category->url(), array( array( 'cid IN ('.implode( ',', $catIDS ).')' ) ), NULL, NULL, ( isset( $categoryOptions->view_videos ) AND $categoryOptions->view_videos ) ? 'view' : 'read' );
        $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' );
		$table->include = array( 'title' );
		$table->mainColumn = 'title';
        $table->limit = ( isset( $categoryOptions->per_page ) AND $categoryOptions->per_page ) ? (int) $categoryOptions->per_page : (int) \IPS\Settings::i()->videos_videos_per_page;

        /* List or thumbnail layout */
        if( !isset( \IPS\Request::i()->cookie['listType'] ) OR \IPS\Request::i()->cookie['listType'] == 'thumb'  )
        {
            $table->classes[] = 'ipsGrid ipsPad';
            $table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoThumbRow' );
        }
        else
        {
            $table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoRow' );
        }

		/* Set default sort */
		if( !\IPS\Request::i()->sortby )
		{
			$table->sortBy = 'date';
		}

        /* Use default category sort */
        if( !\IPS\Request::i()->sortby AND ( isset( $categoryOptions->sort_by ) AND $categoryOptions->sort_by ) )
        {
            $table->sortBy = $categoryOptions->sort_by;
            $table->sortDirection = $categoryOptions->sort_order;
        }

		/* Use default category sort */
		if( !\IPS\Request::i()->sortby AND ( isset( $categoryOptions->sort_by ) AND $categoryOptions->sort_by ) )
        {
            $table->sortBy = $categoryOptions->sort_by;
            $table->sortDirection = $categoryOptions->sort_order;
        }

        /* Sorting table */
        $table->langPrefix = "video_";
        $table->sortOptions['rating'] = 'video_rating';
        $table->sortOptions['approved'] = 'video_status';
    	$table->sortBy = $table->sortBy ?: 'date';
		$table->sortDirection = $table->sortDirection ?: 'desc';

        /* Setup our filters */
		$table->filters	= array(
			'filter_myvideos' => array('video_author_id=?', \IPS\Member::loggedIn()->member_id ),
			'filter_approved' => array('video_status=?', 1 ),
            'filter_unapproved' => array('video_status=?', 0 ),
		 );

		/* Category RSS */
		if( \IPS\Settings::i()->vs_enable_rss AND ( isset( $categoryOptions->cat_rss ) AND $categoryOptions->cat_rss ) )
		{
			/* Show the category link */
			$rssUrl   = \IPS\Http\Url::internal( "app=videos&module=videos&controller=browse&id={$category->_id}&rss=1", 'front', 'videos_category_rss', array( $category->seo_name ) );
			$rssTitle = \IPS\Member::loggedIn()->language()->addToStack( 'latest_videos_category', FALSE, array( 'sprintf' => array( $category->_title ) ) );
			\IPS\Output::i()->rssFeeds[ $rssTitle ] = $rssUrl;

			/* Show RSS feed */
			if ( isset( \IPS\Request::i()->rss ) )
			{
				$document = \IPS\Xml\Rss::newDocument( $category->url(), $rssTitle, $rssTitle );

                /* Go through table items */
				foreach ( $table->getRows( array() ) as $video )
				{
					if ( ! $video->hidden() )
					{
					    $document->addItem( $video->title, $video->url(), $video->description, \IPS\DateTime::ts( $video->date ), $video->tid );
					}
				}

				\IPS\Output::i()->sendOutput( $document->asXML(), 200, 'text/xml' );
			}
		}

        /* Add sidebar block */
        if( $category->children() )
        {
            \IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'browse' )->categoriesBlock( $category );
        }

		/* Online User Location */
		$permissions = $category->permissions();
		\IPS\Session::i()->setLocation( $category->url(), explode( ",", $permissions['perm_view'] ), 'loc_videos_viewing_category', array( "videos_category_{$category->id}" => TRUE ) );

		/* Club or not club nav */
        if ( \IPS\IPS::classUsesTrait( $category, 'IPS\Content\ClubContainer' ) and $club = $category->club() )
        {
            \IPS\core\FrontNavigation::$clubTabActive = TRUE;
            \IPS\Output::i()->breadcrumb = array();
            \IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=clubs&controller=directory', 'front', 'clubs_list' ), \IPS\Member::loggedIn()->language()->addToStack('module__core_clubs') );
            \IPS\Output::i()->breadcrumb[] = array( $club->url(), $club->name );

            if ( \IPS\Settings::i()->clubs_header == 'sidebar' )
            {
                \IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'clubs', 'core' )->header( $club, $category, 'sidebar' );
            }
        }
        else
        {
            /* Add parent cats to nav */
            foreach( $category->parents() AS $parent )
            {
                \IPS\Output::i()->breadcrumb[] = array( $parent->url(), $parent->_title );
            }
        }

        \IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&id='.$category->_id, 'front', 'videos_category', $category->seo_name ), $category->_title );

        /* Display */
        \IPS\Output::i()->linkTags['canonical'] = (string) $category->url();
        \IPS\Output::i()->title	 = $category->_title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->categoryPage( $category, (string) $table );
	}
    
	/**
	 * List all videos
	 *
	 * @return	void
	 */
	protected function listall()
	{
        /* Set list type cookie */
        if( \IPS\Request::i()->listType && \in_array( \IPS\Request::i()->listType, array( 'list', 'thumb' ) ) )
        {
            \IPS\Request::i()->cookie['listType'] = \IPS\Request::i()->listType;
            \IPS\Request::i()->setCookie( 'listType', \IPS\Request::i()->listType, NULL, FALSE );          
        } 	   
       
        /* Get all videos */        
		$table = new \IPS\Helpers\Table\Content( '\IPS\videos\Video', \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&do=listall', 'front', 'videos_listall' ), array(), NULL, NULL, 'read' );
        $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' ); 
		$table->include = array( 'title' );
		$table->mainColumn = 'title';	
        $table->limit = (int) \IPS\Settings::i()->videos_videos_per_page;        
                
        /* List or thumbnail layout */
        if( !isset( \IPS\Request::i()->cookie['listType'] ) OR \IPS\Request::i()->cookie['listType'] == 'thumb'  )
        {
            $table->classes[] = 'ipsGrid ipsPad';
            $table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoThumbRow' );    
        }
        else
        {
            $table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoRow' );    
        }              
        
		/* Set default sort */
		if( !\IPS\Request::i()->sortby )
		{
			$table->sortBy = 'date';
		}        

        /* Sorting table */
        $table->langPrefix = "video_";
        $table->sortOptions['rating'] = 'video_rating';   
        $table->sortOptions['approved'] = 'video_status';     
    	$table->sortBy = $table->sortBy ?: 'date';
		$table->sortDirection = $table->sortDirection ?: 'desc';
        
        /* Setup our filters */
		$table->filters	= array(
			'filter_myvideos' => array('video_author_id=?', \IPS\Member::loggedIn()->member_id ),
			'filter_approved' => array('video_status=?', 1 ),   
            'filter_unapproved' => array('video_status=?', 0 ),         
		 );        
     
		/* Display */
        \IPS\Output::i()->linkTags['canonical'] = (string) \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&do=listall', 'front', 'videos_listall' );
		\IPS\Output::i()->title	 = \IPS\Member::loggedIn()->language()->addToStack('page__listall');
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&do=listall', 'front', 'videos_listall' ), \IPS\Member::loggedIn()->language()->addToStack('page__listall') );        
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->categoryPage( NULL, (string) $table );       
	}    
    
	/**
	 * Show categories
	 *
	 * @return	void
	 */
	public function categories()
	{
		\IPS\Output::i()->title	 = \IPS\Member::loggedIn()->language()->addToStack('module__videos_categories');
\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&do=categories', 'front', 'videos_categories' ), \IPS\Member::loggedIn()->language()->addToStack('module__videos_categories') );     
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->categories();
	} 
    
	/**
	 * Latest Videos RSS
	 *
	 * @return	void
	 */
	protected function rss()
	{
		if( !\IPS\Settings::i()->vs_enable_rss )
		{
			\IPS\Output::i()->error( 'videos_rss_offline', '', 403 );
		}
        
		/* Show rss feed */
		$document = \IPS\Xml\Rss::newDocument( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse', 'front', 'videos' ), \IPS\Member::loggedIn()->language()->get('latest_videos_main'), \IPS\Member::loggedIn()->language()->get('latest_videos_main') );
		
		foreach ( \IPS\videos\Video::getItemsWithPermission( array() ) as $video )
		{
		    /* Add thumbnail to description */
		    if( $video->thumbnail )
            {
                $thumbImg = "<p><img src='".\IPS\File::get( 'videos_Thumbs', $video->thumbnail )->url."' class='ipsThumb ipsThumb_standard' /></p>";    
                $video->description = $thumbImg . $video->description;
            }
          
			$document->addItem( $video->title, $video->url(), $video->description, \IPS\DateTime::ts( $video->date ), $video->tid );
		}

		\IPS\Output::i()->sendOutput( $document->asXML(), 200, 'text/xml' ); 
	}       
}