<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\modules\front\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 */
	public function execute()
	{    
        parent::execute();  

        /* We need at least 1 category */
        if( !\count( \IPS\videos\Category::roots() ) )
        {      
            \IPS\Output::i()->error( 'no_video_categories_found', '', 404, '' );
        }
        
		/* Latest videos RSS feed link */
		if( \IPS\Settings::i()->vs_enable_rss  )
		{
			\IPS\Output::i()->rssFeeds[ 'latest_videos_main' ] = \IPS\Http\Url::internal( 'app=videos&module=videos&controller=browse&do=rss', 'front', 'videos_rss' );
		}       
        
        /* Add featured toggle */
        \IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'front_index.js', 'videos', 'front' ) );      
    }

	/**
	 * Videos Portal
	 *
	 * @return	void
	 */
	protected function manage()
	{
        /* Sort out order */
        $orderField     = \IPS\Settings::i()->vs_members_videos_box_sort_by ? \IPS\Settings::i()->vs_members_videos_box_sort_by : 'date';
        $orderDirection = \IPS\Settings::i()->vs_members_videos_box_sort_order ? \IPS\Settings::i()->vs_members_videos_box_sort_order : 'desc';

        /* Get featured videos */
        $featured = NULL;
        if( \IPS\Settings::i()->vs_featured_videos )
        {
            $featured = iterator_to_array( \IPS\videos\Video::featured( 10, '_rand' ) );

            foreach ( $featured as $firstFeatured )
            {
                /* Update views for first video */
                \IPS\Db::i()->insert( 'core_view_updates', array(
                    'classname'	=> 'IPS\videos\Video',
                    'id'		=> $firstFeatured->tid
                ) );
                break;
            }
        }

        /* Add sidebar block */
        \IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'browse' )->categoriesBlock();

        /* Set Online Location */
        $permissions = \IPS\Dispatcher::i()->module->permissions();
        \IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index', 'front', 'videos' ), explode( ',', $permissions['perm_view'] ), 'loc_videos_index' );

        /* Setup index tabs */
        if( \IPS\Settings::i()->vs_trending_videos )
        {
            $tabs['trending'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'videos_tab_trending' ),
                                       'url'   => \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=trending', 'front', 'videos_tab_trending' )
                                     );
        }

        if( \IPS\Member::loggedIn()->member_id )
        {
            $tabs['following'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'videos_tab_following' ),
                                        'url'   => \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=following', 'front', 'videos_tab_following' )
            );
        }

        $tabs['new'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'videos_tab_new' ),
                                   'url'   => \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=new', 'front', 'videos_tab_new' )
        );
        $tabs['featured'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'videos_tab_featured' ),
                                   'url'   => \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=featured', 'front', 'videos_tab_featured' )
        );

        /* Get category tabs */
        $categoryTabs = explode( ',', \IPS\Settings::i()->vs_category_tabs );

        if( \is_array( $categoryTabs ) AND \count( $categoryTabs ) )
        {
            foreach( $categoryTabs as $tab )
            {
                /* Get category */
                try
                {
                    $category = \IPS\videos\Category::loadAndCheckPerms( $tab );
                }
                catch ( \OutOfRangeException $e )
                {
                    continue;
                }

                /* Add categories to tab list */
                $tabs[ 'category_'.$category->_id ] = array( 'title' => $category->_title,
                                                             'url'   => \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=category&category='.$category->_id, 'front', 'videos_tab_category', $category->seo_name )
                );
            }
        }

        /* Check category tab valid first */
        if( \IPS\Request::i()->tab == 'category' AND \IPS\Request::i()->category AND isset( $tabs[ 'category_'.\IPS\Request::i()->category ] ) AND $tabs[ 'category_'.\IPS\Request::i()->category ] )
        {
            /* Add category id */
            $tab = \IPS\Request::i()->tab.'_'.\IPS\Request::i()->category;
        }

        /* Fall back to default tab? */
        elseif( !isset( \IPS\Request::i()->tab ) or !array_key_exists( \IPS\Request::i()->tab, $tabs ) )
        {
            /* Get default tab and check that trending and followed can be accessed */
            $defaultTab = \IPS\Settings::i()->vs_default_tab;
            $defaultTab = ( ( $defaultTab == 'trending' AND \IPS\Settings::i()->vs_trending_videos ) OR ( $defaultTab == 'following' AND \IPS\Member::loggedIn()->member_id ) ) ? $defaultTab : 'new';

            $tab = $defaultTab;
        }
        /* Everything else */
        else
        {
            $tab = \IPS\Request::i()->tab;
        }

        /* Work out the tab content */
        $tabContent = '';
        if( mb_strpos( $tab, 'category' ) === 0 AND isset( $tabs[ $tab ] ) AND $tabs[ $tab ] )
        {
            /* Get category videos */
            $table = new \IPS\Helpers\Table\Content( 'IPS\videos\Video', $tabs[ $tab ]['url'], array( 'cid=? AND video_status=1', \IPS\Request::i()->category ), NULL, FALSE, 'read', FALSE );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoThumbRow' );
            $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoTable' );
            $table->limit         = ( \IPS\Settings::i()->vs_members_videos_box_limit ) ? (int) \IPS\Settings::i()->vs_members_videos_box_limit : 12;
            $table->noModerate    = TRUE;

            if( !\IPS\Request::i()->sortby )
            {
                $table->sortBy = $orderField;
            }

            /* Output 'category videos' to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->videosTab( $tab, (string) $table );
        }
        elseif( $tab == 'new' )
        {
            /* Get new videos */
$table = new \IPS\Helpers\Table\Content( 'IPS\videos\Video', \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=new', 'front', 'videos_tab_new' ), NULL, NULL, FALSE, 'read', FALSE );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoThumbRow' );
            $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoTable' );
            $table->limit         = ( \IPS\Settings::i()->vs_members_videos_box_limit ) ? (int) \IPS\Settings::i()->vs_members_videos_box_limit : 12;
            $table->noModerate    = TRUE;

            if( !\IPS\Request::i()->sortby )
            {
                $table->sortBy = $orderField;
            }

            /* Output 'new videos' to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->videosTab( $tab, (string) $table );
        }
        elseif( $tab == 'trending' AND \IPS\Settings::i()->vs_trending_videos )
        {
            /* Get trending videos */
$table = new \IPS\Helpers\Table\Content( 'IPS\videos\Video', \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=trending', 'front', 'videos_tab_trending' ), NULL, NULL, FALSE, 'read', FALSE );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoThumbRow' );
            $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoTable' );
            $table->limit         = ( \IPS\Settings::i()->vs_trending_limit ) ? (int) \IPS\Settings::i()->vs_trending_limit : 12;
            $table->noModerate    = TRUE;
            $table->sortOptions   = array( 'video_trending_views' );
            $table->sortBy        = 'video_trending_views';
            $table->sortDirection = 'desc';
            $table->where[]       = array( 'video_trending_views>0' );

            /* Output 'trending videos' to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->videosTab( $tab, (string) $table );
        }
        elseif( $tab == 'following' AND \IPS\Member::loggedIn()->member_id )
        {
            /* Get list of followers */
            $memberIDS = iterator_to_array( \IPS\Db::i()->select( 'follow_rel_id', 'core_follow', array( array( 'follow_app=? AND follow_area=? AND follow_member_id=?', 'core', 'member', \IPS\Member::loggedIn()->member_id ) ) ) );

            /* Get followed videos */
            $table = new \IPS\Helpers\Table\Content( 'IPS\videos\Video', \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=following', 'front', 'videos_tab_following' ), NULL, NULL, FALSE, 'read', FALSE );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoThumbRow' );
            $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoTable' );
            $table->limit         = ( \IPS\Settings::i()->vs_members_videos_box_limit ) ? (int) \IPS\Settings::i()->vs_members_videos_box_limit : 12;
            $table->noModerate    = TRUE;
            $table->where[]       = array( \IPS\Db::i()->in( 'video_author_id', $memberIDS ) );

            if( !\IPS\Request::i()->sortby )
            {
                $table->sortBy = $orderField;
            }

            /* Output 'new videos' to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->videosTab( $tab, (string) $table );
        }
        elseif( $tab == 'featured' )
        {
            /* Get trending videos */
            $table = new \IPS\Helpers\Table\Content( 'IPS\videos\Video', \IPS\Http\Url::internal( 'app=videos&module=videos&controller=index&tab=featured', 'front', 'videos_tab_featured' ), NULL, NULL, FALSE, 'read', FALSE );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoThumbRow' );
            $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' ), 'videoTable' );
            $table->limit         = ( \IPS\Settings::i()->vs_members_videos_box_limit ) ? (int) \IPS\Settings::i()->vs_members_videos_box_limit : 12;
            $table->noModerate    = TRUE;
            $table->sortOptions   = array();
            $table->where[]       = array( 'featured=1' );

            /* Output 'featured videos' to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->videosTab( $tab, (string) $table );
        }

        /* Display tab request on ajax */
        if( \IPS\Request::i()->isAjax() and isset( \IPS\Request::i()->tab ) )
        {
            \IPS\Output::i()->sendOutput( $tabContent );
        }

        /* Display page */
        \IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'module__videos_videos' );
        \IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->index( $featured, $tabs, $tab, $tabContent );
	}
}