<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\modules\front\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Submit Controller
 */
class _submit extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Output::i()->sidebar['enabled'] = FALSE;

		parent::execute();

		/* Check group add perms */
		if( !\IPS\Member::loggedIn()->group['g_vs_add_video'] )
        {
            \IPS\Output::i()->error( 'video_no_category_add_perms', '', 403, '' );
        }

        /* Any max videos limit for member? */
        if ( !\IPS\videos\Video::canCreate( \IPS\Member::loggedIn(), NULL, TRUE ) )
        {
            \IPS\Output::i()->error( 'no_module_permission', '', 403, '' );
        }
   
	}

	/**
	 * Choose category
	 */
	protected function manage()
	{
	    /* Can add to any category? */
        if( !\IPS\videos\Category::canOnAny('add') )
        {
            \IPS\Output::i()->error( 'video_no_category_add_perms', '', 403, '' );     
        }
       
        /* Select cat form */
        $form = new \IPS\Helpers\Form( 'form', NULL, NULL, array( 'data-bypassValidation' => true ) );
		$form->class = 'ipsForm_vertical ';

        $form->actionButtons = array();

        /* Prioritize quick add */
        if( !\IPS\Settings::i()->vs_quick_add OR ( \IPS\Settings::i()->vs_quick_add AND !\IPS\Settings::i()->vs_quick_add_prefer ) )
        {
            $form->addButton( 'videos_continue', 'submit', NULL, 'ipsButton ipsButton_primary' );
        }

        /* Add quick add button */
        if( \IPS\Settings::i()->vs_quick_add )
        {
            $form->addButton( 'videos_quick_add', 'submit', NULL, 'ipsButton ipsButton_secondary', array( 'name' => 'quickAddButton', 'value' => 1 ) );
        }

		$form->add( new \IPS\Helpers\Form\Node( 'video_category', \IPS\Request::i()->category, TRUE, array(
			'url'					=> \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit', 'front', 'videos_add' ),
			'class'					=> 'IPS\videos\Category',
            'permissionCheck'		=> function( $node )
			{ 
			    /* Cat only? */
                if( $node->cat_only )
                {
                    return FALSE;
                }			 
             
                /* Can add? */
				if ( $node->can( 'add' ) )
				{
                    return TRUE;
				}
				
				return NULL;
			}
		) ) );
        
        /* Setup quick add support */
        if( \IPS\Settings::i()->vs_quick_add )
        {
            $form->add( new \IPS\Helpers\Form\Url( 'videos_media', NULL, FALSE, array( 'placeholder' => 'https://www.youtube.com/watch?v=Oh4i5ECWblo' ), function( $val )
            {
                /* Check for duplicate media urls? */
                if( \IPS\Settings::i()->vs_duplicate_media_url )
                {
                    /* Default match check */
                    $duplicateWhere = array( array( 'video_data=?', (string) $val ) );

                    /* Check url against YouTube regex */
                    preg_match( "/^(?:http(?:s)?:\/\/)?(?:www\.)?(?:m\.)?(?:youtu\.be\/|youtube\.com\/(?:(?:watch)?\?(?:.*&)?v(?:i)?=|(?:embed|v|vi|user)\/))([^\?&\"'>]+)/", $val, $matches );

                    /* Perform id check for YouTube videos */
                    if( isset( $matches[1] ) AND $matches[1] )
                    {
                        $duplicateWhere = array( array( "video_data LIKE '%".$matches[1]."%'" ) );
                    }

                    /* Lets check approved videos only */
                    $duplicateWhere[] = array( 'video_status=?', 1 );

                    /* Throw error if any found */
                    $existingCount = \IPS\Db::i()->select( 'video_data', 'videos_videos', $duplicateWhere )->count();

                    if( $existingCount )
                    {
                        throw new \DomainException( 'duplicate_media_url' );
                    }
                }
            }, NULL, NULL, 'videos_media' ) );
        }

		if ( $values = $form->values() )
		{
		    /* Setup next step url */
            $url = \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit&do=submit', 'front', 'videos_add' )->setQueryString( array( 'category' => $values['video_category']->_id ) );

            /* Is quick add enabled? */
            if ( \IPS\Settings::i()->vs_quick_add AND $values['videos_media'] )
            {
                /* Add video right now? */
                if( isset( \IPS\Request::i()->quickAddButton ) and \IPS\Request::i()->quickAddButton )
                {
                    /* Try and get media data */
                    try
                    {
                        $mediaData = \IPS\videos\MediaSite::loadDataFromUrl( $values['videos_media'] );
                    }
                    catch( \OutOfRangeException $e ){}

                    /* We at least need a confirmed title before adding */
                    if( isset( $mediaData['title'] ) AND $mediaData['title'] )
                    {
                        $video = \IPS\videos\Video::createItem( \IPS\Member::loggedIn(), NULL, \IPS\DateTime::ts( time() ), $values['video_category'] );
                        $video->title       = $mediaData['title'];
                        $video->description = \IPS\Text\Parser::parseStatic( $mediaData['description'], TRUE, NULL, \IPS\Member::loggedIn(), 'videos_Videos', TRUE, !(bool) \IPS\Member::loggedIn()->group['g_dohtml'] );
                        $video->video_data  = $values['videos_media'];
                        $video->video_type  = 'media_url';
                        $video->save();
                        $video->generateThumbnail();
                        $video->generateEmbed();

                        /* Any tags to save? */
                        if( $values['video_category']->tags_enable AND isset( $mediaData['tags'] ) AND \count( $mediaData['tags'] ) )
                        {
                            $tags = $mediaData['tags'];

                            /* Limit tags? */
                            if( \IPS\Settings::i()->tags_max )
                            {
                                $tags = \array_slice( $tags, 0, \IPS\Settings::i()->tags_max );
                            }

                            /* Add tags to video */
                            $video->setTags( $tags );
                        }

                        /* Send them to new video */
                        \IPS\Output::i()->redirect( $video->url() );
                    }
                }

                /* Pass url to add video form */
                if( $values['videos_media'] )
                {
                    $url = $url->setQueryString( array( 'media' => base64_encode( (string) $values['videos_media'] ) ) );
                }
            }

            \IPS\Output::i()->redirect( $url );
		}

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'videos_select_category' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->categorySelect( $form );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'videos_select_category' ) );
	}

	/**
	 * Submit videos
	 *
	 * @return	void
	 */
	protected function submit()
	{	
        /* Get category */
		try
		{
			$category = \IPS\videos\Category::loadAndCheckPerms( \IPS\Request::i()->category );
            
            /* Is cat only? */
            if( $category->cat_only )
            {
                \IPS\Output::i()->error( 'cat_only_error', '', 403, '' );   
            }

		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit', 'front', 'videos_add' ) );
		}

        /* Club or normal story? */
        if ( \IPS\IPS::classUsesTrait( $category, 'IPS\Content\ClubContainer' ) and $club = $category->club() )
        {
            \IPS\core\FrontNavigation::$clubTabActive = TRUE;
            \IPS\Output::i()->breadcrumb = array();
            \IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=clubs&controller=directory', 'front', 'clubs_list' ), \IPS\Member::loggedIn()->language()->addToStack('module__core_clubs') );
            \IPS\Output::i()->breadcrumb[] = array( $club->url(), $club->name );

            if ( \IPS\Settings::i()->clubs_header == 'sidebar' )
            {
                \IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'clubs', 'core' )->header( $club, $category, 'sidebar' );
            }
        }

        /* Show form */
		\IPS\Output::i()->title = $category->_title;
		\IPS\Output::i()->breadcrumb[] = array( NULL, $category->_title );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('submit')->formPage( $category, \IPS\videos\Video::create( $category ) ); 	   
	}

    /**
     * Submit from forum post
     *
     * @return	void
     */
    protected function postVideo()
    {
        \IPS\Session::i()->csrfCheck();

        /* No forums, no function */
        if( !\IPS\Application::appIsEnabled('forums') )
        {
            \IPS\Output::i()->error( 'video_no_category_add_perms', '', 403, '' );
        }

        /* Can add to any category? */
        if( !\IPS\videos\Category::canOnAny('add') )
        {
            \IPS\Output::i()->error( 'video_no_category_add_perms', '', 403, '' );
        }

        /* We need a post */
        if( !\IPS\Request::i()->post )
        {
            \IPS\Output::i()->error( 'videos_no_forum_post', '', 403, '' );
        }

        /* Get post */
        try
        {
            $post = \IPS\forums\Topic\Post::loadAndCheckPerms( \IPS\Request::i()->post );
        }
        catch( \Exception $e )
        {
            \IPS\Output::i()->error( 'videos_no_forum_post', '', 403, '' );
        }

        /* Start domdocument, force it to UTF-8 */
        libxml_use_internal_errors(TRUE);
        $doc = new \DOMDocument( '1.0', 'UTF-8' );
        @$doc->loadHTML( "<head><meta http-equiv='Content-Type' content='text/html; charset=utf-8'/></head>" . $post->post );

        /* Setup media urls */
        $mediaUrls = array();

        /* Go through all iframes */
        $iframes = $doc->getElementsByTagName('iframe');

        foreach( $iframes as $iframe )
        {
            /* Is url? */
            if( filter_var( $iframe->getAttribute('data-embed-src'), FILTER_VALIDATE_URL ) )
            {
                $url = $iframe->getAttribute('data-embed-src');
            }
            if( filter_var( $iframe->getAttribute('src'), FILTER_VALIDATE_URL ) )
            {
                $url = $iframe->getAttribute('src');
            }

            /* Grab Youtube url */
            if( preg_match("/^(?:http(?:s)?:\/\/)?(?:www\.)?(?:m\.)?(?:youtu\.be\/|youtube\.com\/(?:(?:watch)?\?(?:.*&)?v(?:i)?=|(?:embed|v|vi|user)\/))([^\?&\"'>]+)/", $url, $vids ) )
            {
                if( isset( $vids[1] ) AND $vids[1] )
                {
                    $url = "https://www.youtube.com/watch?v=".$vids[1];
                }
            }

            /* Grab Vimeo url */
            if( preg_match("~https://player.vimeo.com/video/(.*?)\?~si", $url, $vids ) )
            {
                if( isset( $vids[1] ) AND $vids[1] )
                {
                    $url = "https://vimeo.com/".$vids[1];
                }
            }

            /* Can we use media url? */
            try
            {
                $this->embed = \IPS\Text\Parser::embeddableMedia( \IPS\Http\Url::createFromString( $url, FALSE, TRUE ), TRUE );
                $mediaUrls[] = $url;
            }
            catch( \Exception $e ){}
        }

        /* No media urls, not much point here */
        if( !\count( $mediaUrls ) )
        {
            \IPS\Output::i()->error( 'videos_no_forum_post_urls', '', 403, '' );
        }

        /* Select cat form */
        $form = new \IPS\Helpers\Form( 'form', NULL, NULL, array( 'data-bypassValidation' => true ) );
        $form->class = 'ipsForm_vertical ';

        $form->actionButtons = array();
        $form->addButton( 'videos_continue', 'submit', NULL, 'ipsButton ipsButton_primary' );

        /* Add quick add button */
        if( \IPS\Settings::i()->vs_quick_add )
        {
            $form->addButton( 'videos_quick_add', 'submit', NULL, 'ipsButton ipsButton_secondary', array( 'name' => 'quickAddButton', 'value' => 1 ) );
        }

        $form->add( new \IPS\Helpers\Form\Node( 'video_category', NULL, TRUE, array(
            'url'					=> \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit', 'front', 'videos_add' ),
            'class'					=> 'IPS\videos\Category',
            'permissionCheck'		=> function( $node )
            {
                /* Cat only? */
                if( $node->cat_only )
                {
                    return FALSE;
                }

                /* Can add? */
                if ( $node->can( 'add' ) )
                {
                    return TRUE;
                }

                return NULL;
            }
        ) ) );

        if ( $values = $form->values() )
        {
            /* Setup next step url */
            $submitUrl = \IPS\Http\Url::internal( 'app=videos&module=videos&controller=submit&do=submit', 'front', 'videos_add' )->setQueryString( array( 'category' => $values['video_category']->_id ) );

            /* Is quick add enabled? */
            if ( \IPS\Settings::i()->vs_quick_add )
            {
                /* Add video right now? */
                if( isset( \IPS\Request::i()->quickAddButton ) and \IPS\Request::i()->quickAddButton )
                {
                    $firstVideo  = NULL;
                    $extraVideos = array();

                    foreach( $mediaUrls as $idx => $url )
                    {
                        /* Use the first url as main video */
                        if( $idx == 0 )
                        {
                            $firstVideo = $url;
                            continue;
                        }

                        /* Add url for extra videos */
                        $extraVideos[] = array( 'key' => \IPS\Member::loggedIn()->language()->get( 'extra_videos__video' ) . $idx, 'value' => $url );
                    }

                    /* Try and get media data */
                    try
                    {
                        $mediaData = \IPS\videos\MediaSite::loadDataFromUrl( $firstVideo );
                    }
                    catch( \OutOfRangeException $e ){}

                    /* We at least need a confirmed title before adding */
                    if( $firstVideo AND isset( $mediaData['title'] ) AND $mediaData['title'] )
                    {
                        $video = \IPS\videos\Video::createItem( \IPS\Member::loggedIn(), NULL, \IPS\DateTime::ts( time() ), $values['video_category'] );
                        $video->title       = $mediaData['title'];
                        $video->description = \IPS\Text\Parser::parseStatic( $mediaData['description'], TRUE, NULL, \IPS\Member::loggedIn(), 'videos_Videos', TRUE, !(bool) \IPS\Member::loggedIn()->group['g_dohtml'] );
                        $video->video_data  = $firstVideo;
                        $video->video_type  = 'media_url';

                        /* Add extra videos */
                        if( \count( $extraVideos ) )
                        {
                            $video->extra_videos = json_encode( $extraVideos );
                        }

                        $video->save();
                        $video->generateThumbnail();
                        $video->generateEmbed();

                        /* Any tags to save? */
                        if( $values['video_category']->tags_enable AND isset( $mediaData['tags'] ) AND \count( $mediaData['tags'] ) )
                        {
                            $tags = $mediaData['tags'];

                            /* Limit tags? */
                            if( \IPS\Settings::i()->tags_max )
                            {
                                $tags = \array_slice( $tags, 0, \IPS\Settings::i()->tags_max );
                            }

                            /* Add tags to video */
                            $video->setTags( $tags );
                        }

                        /* Send them to new video */
                        \IPS\Output::i()->redirect( $video->url() );
                    }
                }

                /* Pass url(s) to add video form */
                $submitUrl = $submitUrl->setQueryString( array( 'media' => base64_encode( implode( ',', $mediaUrls ) ) ) );
            }

            \IPS\Output::i()->redirect( $submitUrl );
        }

        \IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'videos_select_category' );
        \IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->categorySelect( $form );
        \IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'videos_select_category' ) );
    }
}