<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\modules\front\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * view
 */
class _view extends \IPS\Content\Controller
{
	/**
	 * [Content\Controller]	Class
	 */
	protected static $contentModel = 'IPS\videos\Video';       
    
	/**
	 * View Video Page
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Init */
		parent::manage();

	    /* Get video entry */
		try
		{
			$video = \IPS\videos\Video::loadAndCheckPerms( \IPS\Request::i()->id );

            /* Can view video? */
			if ( !$video->canView( \IPS\Member::loggedIn() ) )
			{
				\IPS\Output::i()->error( 'node_error', '', 403, '' );
			}

            /* Add thumbnail to meta */
			if( $video->thumbnail )
			{
				\IPS\Output::i()->metaTags['og:image'] = \IPS\File::get( 'videos_Thumbs', $video->thumbnail )->url;
                \IPS\Output::i()->metaTags['meta_imagesrc'] = \IPS\File::get( 'videos_Thumbs', $video->thumbnail )->url;
                \IPS\Output::i()->metaTags['og:image:width'] = "398";
                \IPS\Output::i()->metaTags['og:image:height'] = "224";
			} 

            if( $video->video_type == 'media_url' OR !$video->video_type )
            {
                \IPS\Output::i()->metaTags['og:type'] = "video";
                \IPS\Output::i()->metaTags['og:video'] = $video->video_data;
                \IPS\Output::i()->metaTags['og:video:url'] = $video->video_data;
                \IPS\Output::i()->metaTags['og:video:type'] = "application/x-shockwave-flash";
                \IPS\Output::i()->metaTags['og:video:width'] = "398";
                \IPS\Output::i()->metaTags['og:video:height'] = "224";
                \IPS\Output::i()->metaTags['medium'] = "video";
            }
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '', 404, '' );
		}
        
        /* Setup featured switcher return */
        if( \IPS\Request::i()->featured AND \IPS\Request::i()->isAjax() )
        {
            /* Update views */
            \IPS\Db::i()->insert( 'core_view_updates', array(
                'classname'	=> static::$contentModel,
                'id'		=> $video->tid
            ) );

            \IPS\Output::i()->json( array( 'html' => \IPS\Theme::i()->getTemplate( 'browse', 'videos', 'front' )->featuredVideo( $video ) ) );
        }

		$previous = NULL;
		$next     = NULL;

		/* Previous chapter */
		try
		{
			$previous = \IPS\Db::i()->select(
					'*',
					'videos_videos',
					array( 'cid=? AND tid<? AND video_status=1', $video->container()->_id, $video->tid ),
					'tid DESC'
					,1
					)->first();

			$previous = \IPS\videos\Video::constructFromData( $previous );

            /* Can view previous? */
            if( !$previous->canView() )
            {
                $previous= NULL;
            }
		}
		catch ( \UnderflowException $e ) {}

		/* Next chapter */
		try
		{
			$next = \IPS\Db::i()->select(
					'*',
					'videos_videos',
					array( 'cid=? AND tid>? AND video_status=1', $video->container()->_id, $video->tid ),
					'tid ASC'
					,1
					)->first();

			$next = \IPS\videos\Video::constructFromData( $next );

            /* Can view next? */
            if( !$next->canView() )
            {
                $next = NULL;
            }
		}
		catch ( \UnderflowException $e ) {}

		/* Online User Location */
		\IPS\Session::i()->setLocation( $video->url(), $video->onlineListPermissions(), 'loc_videos_viewing_video', array( $video->title => FALSE ) );

		/* Update trending views? */
		if( \IPS\Settings::i()->vs_trending_videos )
        {
            $video->video_trending_views++;
            $video->save();
        }

        /* Add Video.js files */
        if( $video->video_type == 'media_upload' OR $video->video_type == 'media_upload_url' )
        {
            \IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, array( \IPS\Http\Url::external( 'https://vjs.zencdn.net/6.6.3/video-js.css' ) ) );
            \IPS\Output::i()->jsFiles  = array_merge( \IPS\Output::i()->jsFiles, array( \IPS\Http\Url::external( 'https://vjs.zencdn.net/ie8/1.1.2/videojs-ie8.min.js' ) ) );
            \IPS\Output::i()->jsFiles  = array_merge( \IPS\Output::i()->jsFiles, array( \IPS\Http\Url::external( 'https://vjs.zencdn.net/6.6.3/video.js' ) ) );
        }

        /* Process any relative urls for upload urls */
        if( $video->video_type == 'media_upload_url' )
        {
            /* Change board url tag to actual url */
            if( mb_substr( $video->video_data, 0, 11 ) === '%BOARD_URL%' )
            {
                $video->video_data = str_replace( '%BOARD_URL%', \IPS\Http\Url::internal( '' ), $video->video_data );
            }
        }

        /* Apply url paramaters to Youtube embeds */
        if( ( $video->video_type == 'media_url' OR !$video->video_type ) AND mb_strstr( $video->embed, 'youtube.com/embed/' ) )
        {
            $urlParam    = '';
            $videoParams = ( \IPS\Settings::i()->vs_youtube_params ) ? json_decode( \IPS\Settings::i()->vs_youtube_params, TRUE ) : array();

            /* Go through all paramaters */
            foreach( $videoParams as $param )
            {
                $urlParam = "&".$param['key']."=".$param['value'];

                /* Append url paramater */
                $video->embed = str_replace( '?feature=oembed', '?feature=oembed'.$urlParam, $video->embed );
            }

            /* Enable privacy mode */
            if( \IPS\Settings::i()->vs_youtube_privacy )
            {
                $video->embed = str_replace( 'youtube.com', 'youtube-nocookie.com', $video->embed );
                $video->embed = str_replace( '//youtube-nocookie.com', '//www.youtube-nocookie.com', $video->embed );
            }
        }

        /* Print video entry */
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->view( $video, $previous, $next );
	}

	/**
	 * Change Author
	 *
	 * @return	void
	 */
	public function changeAuthor()
	{
	    /* Get video entry */
		try
		{
			$video = \IPS\videos\Video::loadAndCheckPerms( \IPS\Request::i()->id );

            /* Can view video? */
			if ( !$video->canView( \IPS\Member::loggedIn() ) )
			{
				\IPS\Output::i()->error( 'node_error', '', 403, '' );
			}
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '', 404, '' );
		}

		/* Permission check */
		if ( !$video->canChangeAuthor() )
		{
			\IPS\Output::i()->error( 'no_module_permission', '', 403, '' );
		}

		/* Build form */
		$form = new \IPS\Helpers\Form;
		$form->add( new \IPS\Helpers\Form\Member( 'author', NULL, TRUE ) );
		$form->class .= 'ipsForm_vertical';

		/* Handle submissions */
		if ( $values = $form->values() )
		{
			$video->changeAuthor( $values['author'] );
			\IPS\Output::i()->redirect( $video->url() );
		}

		/* Display form */
		\IPS\Output::i()->output = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );;
	}
}