<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */
 
namespace IPS\videos;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Media Site Node
 */
class _MediaSite extends \IPS\Node\Model
{
	protected static $multitons;    
    public static $databasePrefix = 'site_';
    protected static $databaseIdFields = array( 'site_id', 'site_file' );
    public static $databaseColumnId = 'id';
	public static $databaseTable = 'videos_sites';
	public static $nodeTitle = 'module__sites';
	public static $titleLangPrefix = 'videos_mediasite_';
	public static $modalForms = TRUE;

    /**
     * Default oEmbed Sites
     * @return	array
     */
    protected static function oembedSites()
    {
        $services = array(
            'youtube.com'					=> array( 'https://www.youtube.com/oembed', static::EMBED_VIDEO ),
            'm.youtube.com'					=> array( 'https://www.youtube.com/oembed', static::EMBED_VIDEO ),
            'youtu.be'						=> array( 'https://www.youtube.com/oembed', static::EMBED_VIDEO ),
            'flickr.com'					=> array( 'https://www.flickr.com/services/oembed/', static::EMBED_IMAGE ),
            'flic.kr'						=> array( 'https://www.flickr.com/services/oembed/', static::EMBED_IMAGE ),
            'hulu.com'						=> array( 'http://www.hulu.com/api/oembed.json', static::EMBED_VIDEO ),
            'vimeo.com'						=> array( 'https://vimeo.com/api/oembed.json', static::EMBED_VIDEO ),
            'collegehumor.com'				=> array( 'http://www.collegehumor.com/oembed.json', static::EMBED_VIDEO ),
            'twitter.com'					=> array( 'https://publish.twitter.com/oembed', static::EMBED_TWEET ),
            'mobile.twitter.com'			=> array( 'https://publish.twitter.com/oembed', static::EMBED_TWEET ),
            'soundcloud.com'				=> array( 'https://soundcloud.com/oembed', static::EMBED_VIDEO ),
            'open.spotify.com'				=> array( 'https://embed.spotify.com/oembed', static::EMBED_VIDEO ),
            'play.spotify.com'				=> array( 'https://embed.spotify.com/oembed', static::EMBED_VIDEO ),
            'ted.com'						=> array( 'https://www.ted.com/services/v1/oembed', static::EMBED_VIDEO ),
            'vine.co'						=> array( 'https://vine.co/oembed.json', static::EMBED_VIDEO ),
            'dailymotion.com'				=> array( 'https://www.dailymotion.com/services/oembed', static::EMBED_VIDEO ),
            'coub.com'						=> array( 'http://coub.com/api/oembed.json' . ( \IPS\Settings::i()->max_video_width ? ( '?maxwidth=' . \IPS\Settings::i()->max_video_width ) : '' ), static::EMBED_VIDEO ),
            '*.deviantart.com'				=> array( 'https://backend.deviantart.com/oembed', static::EMBED_IMAGE ),
            'docs.com'						=> array( 'http://docs.com/api/oembed', static::EMBED_LINK ),
            'funnyordie.com'				=> array( 'http://www.funnyordie.com/oembed.json', static::EMBED_VIDEO ),
            'gettyimages.com'				=> array( 'http://embed.gettyimages.com/oembed', static::EMBED_IMAGE ),
            'ifixit.com'					=> array( 'http://www.ifixit.com/Embed', static::EMBED_LINK ),
            'kickstarter.com'				=> array( 'http://www.kickstarter.com/services/oembed', static::EMBED_LINK ),
            'meetup.com'					=> array( 'https://api.meetup.com/oembed', static::EMBED_LINK ),
            'mixcloud.com'					=> array( 'https://www.mixcloud.com/oembed/', static::EMBED_VIDEO ),
            'mix.office.com'				=> array( 'https://mix.office.com/oembed', static::EMBED_VIDEO ),
            'reddit.com'					=> array( 'http://www.reddit.com/oembed', static::EMBED_LINK ),
            'reverbnation.com'				=> array( 'https://www.reverbnation.com/oembed', static::EMBED_VIDEO ),
            'screencast.com'				=> array( 'https://api.screencast.com/external/oembed', static::EMBED_IMAGE ),
            'slideshare.net'				=> array( 'http://www.slideshare.net/api/oembed/2', static::EMBED_VIDEO ),
            '*.smugmug.com'					=> array( 'http://api.smugmug.com/services/oembed', static::EMBED_IMAGE ),
            'ustream.tv'					=> array( 'https://www.ustream.tv/oembed', static::EMBED_VIDEO ),
            '*.wistia.com'					=> array( 'http://fast.wistia.com/oembed', static::EMBED_VIDEO ),
            '*.wi.st'						=> array( 'http://fast.wistia.com/oembed', static::EMBED_VIDEO ),
            'tiktok.com'					=> array( 'https://www.tiktok.com/oembed', static::EMBED_VIDEO ),
            'm.tiktok.com'					=> array( 'https://www.tiktok.com/oembed', static::EMBED_VIDEO ),
            'vm.tiktok.com'					=> array( 'https://www.tiktok.com/oembed', static::EMBED_VIDEO ),
        );

        /* Can we support Facebook and Instagram oembeds? */
        if( $token = static::getFacebookToken() )
        {
            $services['instagram.com'] = array(
                array( 'https://graph.facebook.com/v9.0/instagram_oembed?access_token=' . $token, static::EMBED_IMAGE )
            );

            $services['instagr.am'] = $services['instagram.com'];

            $services['facebook.com'] = array(
                '(\/.+?\/videos\/|video.php)'	=> array( 'https://graph.facebook.com/v9.0/oembed_video?access_token=' . $token, static::EMBED_VIDEO ),
                '(\/.+?\/posts\/|\/.+?\/activity\/|\/.+?\/photos?\/|photo.php|\/.+?\/media\/|permalink.php|\/.+?\/questions\/|\/.+?\/notes\/|\/.+?\/media\/)'	=> array( 'https://graph.facebook.com/v9.0/oembed_post?access_token=' . $token, static::EMBED_STATUS ),
                0								=> array( 'https://graph.facebook.com/v9.0/oembed_page?access_token=' . $token, static::EMBED_STATUS ),
            );
        }

        return $services;
    }

    /**
     * Get an application token from Facebook for oembed support
     *
     * @return	NULL|string
     */
    protected static function getFacebookToken()
    {
        try
        {
            if( !\IPS\Settings::i()->fb_ig_oembed_token AND \IPS\Settings::i()->fb_ig_oembed_appid AND \IPS\Settings::i()->fb_ig_oembed_appsecret )
            {
                $value = \IPS\Http\Url::external( "https://graph.facebook.com/oauth/access_token?client_id=" . \IPS\Settings::i()->fb_ig_oembed_appid . "&client_secret=" . \IPS\Settings::i()->fb_ig_oembed_appsecret . "&grant_type=client_credentials" )->request()->get()->decodeJson();

                if( !empty( $value['access_token'] ) )
                {
                    \IPS\Settings::i()->changeValues( array( 'fb_ig_oembed_token' => $value['access_token'] ) );
                }
                else
                {
                    \IPS\Log::debug( "Facebook access token could not be retrieved for oembed:\n" . var_export( $value, true ), 'facebook_oembed' );
                }
            }
        }
        catch( \Exception $e )
        {
            \IPS\Log::debug( $e, 'facebook_oembed' );
        }

        return \IPS\Settings::i()->fb_ig_oembed_token ? \IPS\Settings::i()->fb_ig_oembed_token : NULL;
    }

    /**
     * @brief	Embed type constants
     */
    const EMBED_IMAGE	= 1;
    const EMBED_VIDEO	= 2;
    const EMBED_STATUS	= 3;
    const EMBED_TWEET	= 4;
    const EMBED_LINK	= 5;

    /**
     * Construct ActiveRecord from database row
     *
     * @param	array	$data							Row from database table
     * @param	bool	$updateMultitonStoreIfExists	Replace current object in multiton store if it already exists there?
     * @return	static
     */
    public static function constructFromData( $data, $updateMultitonStoreIfExists = TRUE )
    {
        $classname = 'IPS\videos\MediaSite\\' . $data['site_file'];
        if( !class_exists( $classname ) )
        {
            return parent::constructFromData( $data, $updateMultitonStoreIfExists );
        }

        /* Initiate object */
        $obj = new $classname;
        $obj->_new = FALSE;

        /* Import data */
        foreach( $data as $k => $v )
        {
            if( static::$databasePrefix AND mb_strpos( $k, static::$databasePrefix ) === 0 )
            {
                $k = mb_substr( $k, mb_strlen( static::$databasePrefix ) );
            }

            $obj->_data[ $k ] = $v;
        }

        $obj->changed = array();

        if( method_exists( $obj, 'init' ) )
        {
            $obj->init();
        }

        /* Return it */
        return $obj;
    }

    /**
     * Load media tag with url
     *
     */
    public static function loadEmbedFromUrl( $url=NULL )
    {
        $embedMatch = NULL;

        /* Get 'media tag' sites */
        $mediaTags = parent::roots( NULL, NULL, array( 'site_enabled=1 AND site_type=?', 2 ) );

        foreach( $mediaTags as $site )
        {
            /* Check for site match/replace */
            if ( preg_match( "#^" . $site->match . "$#is", $url, $matches ) )
            {
                if( $result = preg_replace( "#^" . $site->match . "$#is", $site->replace, $url ) )
                {
                    /* Save embed match */
                    $embedMatch = $result;
                }
            }
        }

        /* Return any successful embed match */
        return $embedMatch;
    }

    /**
     * Load media data with url
     *
     */
    public static function loadDataFromUrl( $url=NULL )
    {
        /* Can't really do anything here */
        if( !$url )
        {
            return NULL;
        }

        /* Get domain and remove prefix */
        $parsedUrl = parse_url( $url );
        $domain    = $parsedUrl['host'];

        if ( mb_substr( $domain, 0, 4 ) === 'www.' )
        {
            $domain = mb_substr( $domain, 4 );
        }

        /* Remove suffix and get only host name */
        preg_match('/(.*?)((?:\.co)?.[a-z]{2,4})$/i', $domain, $matches );

        if( !isset( $matches[1] ) OR !$matches[1] )
        {
            return NULL;
        }

        /* Return media data */
        try
        {
            $mediaSite = parent::load( $matches[1], 'site_file' );
            return $mediaSite->getMediaData( $url );
        }
        catch ( \OutOfRangeException $e )
        {
            /* Get already combined list */
            $oembedSites = static::oembedSites();

            if ( array_key_exists( $domain, $oembedSites ) )
            {
                try
                {
                    /* Deal with IPB 4.1.16 changes */
                    if( \is_array( $oembedSites[ $domain ] ) )
                    {
                        $oembedUrl = \IPS\Http\Url::external( $oembedSites[ $domain ][0] )->setQueryString( array( 'format' => 'json', 'url' => $url, 'scheme' => ( $parsedUrl['scheme'] === 'https' or \IPS\Request::i()->isSecure() ) ? 'https' : null ) );
                    }
                    else
                    {
                        $oembedUrl = \IPS\Http\Url::external( $oembedSites[ $domain ] )->setQueryString( array( 'format' => 'json', 'url' => $url, 'scheme' => ( $parsedUrl['scheme'] === 'https' or \IPS\Request::i()->isSecure() ) ? 'https' : null ) );
                    }
                    $response = $oembedUrl->request()->get()->decodeJson();

                    if( !isset( $response['type'] ) )
                    {
                        throw new \RuntimeException( 'NO_EMBED_TYPE' );
                    }

                    return array( 'title'       => $response['title'],
                                  'description' => $response['description'] );
                }
                catch ( \Exception $e ) {}
            }

            return NULL;
        }
    }
   
	/**
	 * Set the title
	 *
	 * @param	string	$title	Title
	 * @return	void
	 */
	public function set_title( $title )
	{
		$this->_data['site_name'] = $title;
	}

	public function get__title()
	{
		if ( !$this->id )
		{
			return '';
		}	   
       
        return $this->name;
	}   
    
	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->enabled;
	}

	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		$this->enabled	= $enabled;
	}

    /**
     * [Node] Return the custom badge for each row
     *
     * @return	NULL|array		Null for no badge, or an array of badge data (0 => CSS class type, 1 => language string, 2 => optional raw HTML to show instead of language string)
     */
    protected function get__badge()
    {
        /* Display oEmbed or Media Tag types */
        return array(
            0	=> ( $this->type == 1 ) ? 'ipsBadge ipsBadge_style5' : 'ipsBadge ipsBadge_style3',
            1	=> ( $this->type == 1 ) ? 'vmedia_type__oembed' : 'vmedia_type__tag',
        );
    }
            	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{		   
        /* Display form */       
		$form->addHeader( 'vmedia_site_settings' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vmedia_enabled', $this->enabled ? (bool) $this->enabled : TRUE, FALSE, array() ) );
        $form->add( new \IPS\Helpers\Form\Radio( 'vmedia_type', $this->id ? (int) $this->type : 1, FALSE, array( 'options' => array( 1 => 'vmedia_type__oembed', 2 => 'vmedia_type__tag' ), 'toggles' => array( 1 => array( 'vmedia_host', 'vmedia_oembed' ), 2 => array( 'vmedia_match', 'vmedia_replace' ) ) ) ) );
		$form->add( new \IPS\Helpers\Form\Text( 'vmedia_name', ( $this->name ) ? $this->name : '', TRUE, array(), NULL, NULL, NULL ) );
		$form->add( new \IPS\Helpers\Form\Text( 'vmedia_host', ( $this->host ) ? $this->host : '', TRUE, array(), NULL, NULL, NULL, 'vmedia_host' ) );
		$form->add( new \IPS\Helpers\Form\Url( 'vmedia_url', ( $this->url ) ? $this->url : '', TRUE, array(), NULL, NULL, NULL, 'vmedia_url' ) );
		$form->add( new \IPS\Helpers\Form\Url( 'vmedia_oembed', ( $this->oembed ) ? $this->oembed : '', FALSE, array(), NULL, NULL, NULL, 'vmedia_oembed' ) );
        $form->add( new \IPS\Helpers\Form\Text( 'vmedia_match', $this->id ? $this->match : NULL, TRUE, array(), NULL, NULL, NULL, 'vmedia_match' ) );
        $form->add( new \IPS\Helpers\Form\TextArea( 'vmedia_replace', $this->id ? $this->replace : NULL, TRUE, array(), NULL, NULL, NULL, 'vmedia_replace' ) );

        /* Custom fields */
        $mediaFields   = $this->fields ? json_decode( $this->fields, true ) : array(); /* Messy but lets do this for now */
        $mediaSettings = $this->settings ? json_decode( $this->settings, true ) : array();

        if( \count( $mediaFields ) AND \is_array( $mediaFields ) )
        {
            $form->addHeader( 'videos_media_site_fields' );

            foreach( $mediaFields as $field )
            {
                if( $field['type'] == 'input' )
                {
                    $form->add( new \IPS\Helpers\Form\Text( 'vmedia_' . $field['name'], $this->id ? $mediaSettings[ $field['name'] ] : '', FALSE, array() ) );
                }
                else if( $field['type'] == 'yes_no' )
                {
                    $form->add( new \IPS\Helpers\Form\YesNo( 'vmedia_' . $field['name'], $this->id ? \intval( $mediaSettings[ $field['name'] ] ) : FALSE, TRUE, array() ) );

                }
                else if( $field['type'] == 'textarea' )
                {
                    $form->add( new \IPS\Helpers\Form\TextArea( 'vmedia_' . $field['name'], $this->id ? $mediaSettings[ $field['name'] ] : '', FALSE, array() ) );
                }
                else if( $field['type'] == 'dropdown' )
                {
                    if( \is_array( $field['options'] ) AND \count( $field['options'] ) )
                    {
                        foreach( $field['options'] as $key => $value )
                        {
                            $fieldDropdown[] = array( $key, $value );
                        }
                    }
                }
                else
                {}
            }
        }
 	}
    
	/**
	 * [Node] Format form values from add/edit form for save
	 *
	 * @param	array	$values	Values from the form
	 * @return	array
	 */
	public function formatFormValues( $values )
    {
        foreach ($values as $k => $v) {
            if (mb_substr($k, 0, 7) === 'vmedia_') {
                unset($values[$k]);
                $values[mb_substr($k, 7)] = $v;
            }
        }

        $values['url'] = (isset($values['url']) AND $values['url']) ? (string)$values['url'] : NULL;
        $values['oembed'] = (isset($values['oembed']) AND $values['oembed']) ? (string)$values['oembed'] : NULL;

        /* Go through media site settings and encode */
        if( !$this->_new )
        {
            $mediaSite = parent::load( $this->id );
            $mediaFields = $mediaSite->fields ? json_decode( $mediaSite->fields, true ) : array();

            if( \count($mediaFields) AND \is_array( $mediaFields ) )
            {
                foreach ($mediaFields as $field)
                {
                    $values['settings'][$field['name']] = $values[$field['name']];
                    unset($values[$field['name']]);
                }

                $values['settings'] = json_encode($values['settings']);
            }
        }

		/* Send to parent */
		return $values;
	}


	/**
	 * @brief	Cached URL
	 */
	protected $_url	= NULL;

	/**
	 * Get URL
	 *
	 * @return	\IPS\Http\Url
	 */
	public function url()
	{
		return NULL;
	}  
    
	/**
	 * Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
	}
    
	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'site_name';
		}

		if( $order == '_title' )
		{
			$order	= 'site_name';
		}

		return parent::search( $column, $query, $order, $where );
	}      
}