<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */
 
namespace IPS\videos\MediaSite;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * YouTube media site
 */
class _youtube extends \IPS\videos\MediaSite
{
	/**
	 * Get media data
	 */
	public function getMediaData( $url=NULL )
	{
	    /* Can't really do anything here */
	    if( !$url )
        {
            return NULL;
        }

        /* Get any gateway settings */
        $settings = json_decode( $this->settings, TRUE );

        /* No api details, no media data */
        if( !$settings['youtube_api_url'] OR !$settings['youtube_api_key'] )
        {
            throw new \OutOfRangeException;
            return;
        }

        /* Get video id from url */
        preg_match('%(?:youtube(?:-nocookie)?\.com/(?:[^/]+/.+/|(?:v|e(?:mbed)?)/|.*[?&]v=)|youtu\.be/)([^"&?/ ]{11})%i', $url, $match );
        $youtubeID = $match[1];

        /* Attempt to grab API results */
        try
        {
            $apiUrl = \IPS\Http\Url::external( $settings['youtube_api_url'] )->setQueryString( array( 'id' => $youtubeID, 'key' => $settings['youtube_api_key'], 'part' => 'snippet,statistics' ) );
            $response = $apiUrl->request()->get()->decodeJson();

            return array( 'title' => $response['items'][0]['snippet']['title'],
                'description' => $response['items'][0]['snippet']['description'],
                'date'        => \IPS\DateTime::create( $response['items'][0]['snippet']['publishedAt'] )->getTimestamp(),
                'tags'        => $response['items'][0]['snippet']['tags'],
                'views'       => (int) $response['items'][0]['statistics']['viewCount'] );
        }
        catch ( \Exception $e ){}

        return NULL;
    }
}