<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */
 
namespace IPS\videos\Video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _Thumbnail
{
	public function busy()
	{
	}    
    
	/**
	 * Default oEmbed Sites
	 * @return	array
	 */
	protected static function oembedSites()
	{
        $services = array(
            'youtube.com'					=> array( 'https://www.youtube.com/oembed', static::EMBED_VIDEO ),
            'm.youtube.com'					=> array( 'https://www.youtube.com/oembed', static::EMBED_VIDEO ),
            'youtu.be'						=> array( 'https://www.youtube.com/oembed', static::EMBED_VIDEO ),
            'flickr.com'					=> array( 'https://www.flickr.com/services/oembed/', static::EMBED_IMAGE ),
            'flic.kr'						=> array( 'https://www.flickr.com/services/oembed/', static::EMBED_IMAGE ),
            'hulu.com'						=> array( 'http://www.hulu.com/api/oembed.json', static::EMBED_VIDEO ),
            'vimeo.com'						=> array( 'https://vimeo.com/api/oembed.json', static::EMBED_VIDEO ),
            'collegehumor.com'				=> array( 'http://www.collegehumor.com/oembed.json', static::EMBED_VIDEO ),
            'twitter.com'					=> array( 'https://publish.twitter.com/oembed', static::EMBED_TWEET ),
            'mobile.twitter.com'			=> array( 'https://publish.twitter.com/oembed', static::EMBED_TWEET ),
            'soundcloud.com'				=> array( 'https://soundcloud.com/oembed', static::EMBED_VIDEO ),
            'open.spotify.com'				=> array( 'https://embed.spotify.com/oembed', static::EMBED_VIDEO ),
            'play.spotify.com'				=> array( 'https://embed.spotify.com/oembed', static::EMBED_VIDEO ),
            'ted.com'						=> array( 'https://www.ted.com/services/v1/oembed', static::EMBED_VIDEO ),
            'vine.co'						=> array( 'https://vine.co/oembed.json', static::EMBED_VIDEO ),
            'dailymotion.com'				=> array( 'https://www.dailymotion.com/services/oembed', static::EMBED_VIDEO ),
            'coub.com'						=> array( 'http://coub.com/api/oembed.json' . ( \IPS\Settings::i()->max_video_width ? ( '?maxwidth=' . \IPS\Settings::i()->max_video_width ) : '' ), static::EMBED_VIDEO ),
            '*.deviantart.com'				=> array( 'https://backend.deviantart.com/oembed', static::EMBED_IMAGE ),
            'docs.com'						=> array( 'http://docs.com/api/oembed', static::EMBED_LINK ),
            'funnyordie.com'				=> array( 'http://www.funnyordie.com/oembed.json', static::EMBED_VIDEO ),
            'gettyimages.com'				=> array( 'http://embed.gettyimages.com/oembed', static::EMBED_IMAGE ),
            'ifixit.com'					=> array( 'http://www.ifixit.com/Embed', static::EMBED_LINK ),
            'kickstarter.com'				=> array( 'http://www.kickstarter.com/services/oembed', static::EMBED_LINK ),
            'meetup.com'					=> array( 'https://api.meetup.com/oembed', static::EMBED_LINK ),
            'mixcloud.com'					=> array( 'https://www.mixcloud.com/oembed/', static::EMBED_VIDEO ),
            'mix.office.com'				=> array( 'https://mix.office.com/oembed', static::EMBED_VIDEO ),
            'reddit.com'					=> array( 'http://www.reddit.com/oembed', static::EMBED_LINK ),
            'reverbnation.com'				=> array( 'https://www.reverbnation.com/oembed', static::EMBED_VIDEO ),
            'screencast.com'				=> array( 'https://api.screencast.com/external/oembed', static::EMBED_IMAGE ),
            'slideshare.net'				=> array( 'http://www.slideshare.net/api/oembed/2', static::EMBED_VIDEO ),
            '*.smugmug.com'					=> array( 'http://api.smugmug.com/services/oembed', static::EMBED_IMAGE ),
            'ustream.tv'					=> array( 'https://www.ustream.tv/oembed', static::EMBED_VIDEO ),
            '*.wistia.com'					=> array( 'http://fast.wistia.com/oembed', static::EMBED_VIDEO ),
            '*.wi.st'						=> array( 'http://fast.wistia.com/oembed', static::EMBED_VIDEO ),
            'tiktok.com'					=> array( 'https://www.tiktok.com/oembed', static::EMBED_VIDEO ),
            'm.tiktok.com'					=> array( 'https://www.tiktok.com/oembed', static::EMBED_VIDEO ),
            'vm.tiktok.com'					=> array( 'https://www.tiktok.com/oembed', static::EMBED_VIDEO ),
        );

        /* Can we support Facebook and Instagram oembeds? */
        if( $token = static::getFacebookToken() )
        {
            $services['instagram.com'] = array(
                array( 'https://graph.facebook.com/v9.0/instagram_oembed?access_token=' . $token, static::EMBED_IMAGE )
            );

            $services['instagr.am'] = $services['instagram.com'];

            $services['facebook.com'] = array(
                '(\/.+?\/videos\/|video.php)'	=> array( 'https://graph.facebook.com/v9.0/oembed_video?access_token=' . $token, static::EMBED_VIDEO ),
                '(\/.+?\/posts\/|\/.+?\/activity\/|\/.+?\/photos?\/|photo.php|\/.+?\/media\/|permalink.php|\/.+?\/questions\/|\/.+?\/notes\/|\/.+?\/media\/)'	=> array( 'https://graph.facebook.com/v9.0/oembed_post?access_token=' . $token, static::EMBED_STATUS ),
                0								=> array( 'https://graph.facebook.com/v9.0/oembed_page?access_token=' . $token, static::EMBED_STATUS ),
            );
        }

        return $services;
	}

    /**
     * Get an application token from Facebook for oembed support
     *
     * @return	NULL|string
     */
    protected static function getFacebookToken()
    {
        try
        {
            if( !\IPS\Settings::i()->fb_ig_oembed_token AND \IPS\Settings::i()->fb_ig_oembed_appid AND \IPS\Settings::i()->fb_ig_oembed_appsecret )
            {
                $value = \IPS\Http\Url::external( "https://graph.facebook.com/oauth/access_token?client_id=" . \IPS\Settings::i()->fb_ig_oembed_appid . "&client_secret=" . \IPS\Settings::i()->fb_ig_oembed_appsecret . "&grant_type=client_credentials" )->request()->get()->decodeJson();

                if( !empty( $value['access_token'] ) )
                {
                    \IPS\Settings::i()->changeValues( array( 'fb_ig_oembed_token' => $value['access_token'] ) );
                }
                else
                {
                    \IPS\Log::debug( "Facebook access token could not be retrieved for oembed:\n" . var_export( $value, true ), 'facebook_oembed' );
                }
            }
        }
        catch( \Exception $e )
        {
            \IPS\Log::debug( $e, 'facebook_oembed' );
        }

        return \IPS\Settings::i()->fb_ig_oembed_token ? \IPS\Settings::i()->fb_ig_oembed_token : NULL;
    }
    
	/**
	 * @brief	Embed type constants
	 */
	const EMBED_IMAGE	= 1;
	const EMBED_VIDEO	= 2;
	const EMBED_STATUS	= 3;
	const EMBED_TWEET	= 4;
	const EMBED_LINK	= 5;    
    
	/**
	 * Get thumbnail url from oembed url
	 *
	 * @param	string		$url		The URL
	 * @return	string|null	Thumbnail url, or NULL if URL is not embeddable
	 */
	public static function getThumb( $url )
	{
	    if( !$url )
        {
            return null;
        }

		/* Init */
		$parsedUrl = parse_url( $url );
        
		$domain = $parsedUrl['host'];
		if ( mb_substr( $domain, 0, 4 ) === 'www.' )
		{
			$domain = mb_substr( $domain, 4 );
		}
        
        /* Return now */
		if( !$domain )
		{
			return null;
		}
  		
		/* Get already combined list */
        $oembedSites = static::oembedSites();

        /* Add our custom sites */
        $customSites = array();

        if( \count( $mediaSites = \IPS\videos\MediaSite::roots( NULL, NULL, array( array( 'site_enabled=? AND site_type=1', 1 ) ) ) ) )
        {
            foreach( $mediaSites as $site )
            {
                $customSites[ $site->host ] = array( $site->oembed, static::EMBED_VIDEO );
            }

            /* Return merged site list */
            $oembedSites = array_merge( $customSites, $oembedSites );
        }

        /* Check facebook content */
        if( $domain == 'facebook.com' )
        {
            /* Get video id from query */
            preg_match("~/videos/(?:t\.\d+/)?(\d+)~i", $url, $facebookID);

            if( !isset( $facebookID[1] ) OR !$facebookID[1] )
            {
                return NULL;
            }

            $apiUrl = \IPS\Http\Url::external( "https://graph.facebook.com/{$facebookID[1]}/picture?redirect=false" );
            $response = $apiUrl->request()->get()->decodeJson();

            /* Return if successful */
            return ( isset( $response['data']['url'] ) AND $response['data']['url'] ) ? $response['data']['url'] : NULL;
        }

		if ( array_key_exists( $domain, $oembedSites ) )
		{
			try
			{
                /* Deal with IPB 4.1.16 changes */
                if( \is_array( $oembedSites[ $domain ] ) )
                {
                    $oembedUrl = \IPS\Http\Url::external( $oembedSites[ $domain ][0] )->setQueryString( array( 'format' => 'json', 'url' => $url, 'scheme' => ( $parsedUrl['scheme'] === 'https' or \IPS\Request::i()->isSecure() ) ? 'https' : null ) );
                }
                else
                {
                    $oembedUrl = \IPS\Http\Url::external( $oembedSites[ $domain ] )->setQueryString( array( 'format' => 'json', 'url' => $url, 'scheme' => ( $parsedUrl['scheme'] === 'https' or \IPS\Request::i()->isSecure() ) ? 'https' : null ) );
                }
                $response = $oembedUrl->request()->get()->decodeJson();

				if( !isset( $response['type'] ) )
				{
					throw new \RuntimeException( 'NO_EMBED_TYPE' );
				}

                /* Return if successful */
                return ( isset( $response['thumbnail_url'] ) AND $response['thumbnail_url'] ) ? $response['thumbnail_url'] : NULL;
			}
			catch ( \Exception $e ) {}
		}

		/* Just return nothing */
		return NULL;
	}                 
}
