<?php
/**
 * @package		Videos
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\videos\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * importVideos Task
 */
class _importVideos extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
        $timeCheck = new \IPS\DateTime;
        $timeCheck->sub( new \DateInterval( 'PT50M' ) );

        $this->runUntilTimeout(function() use ( $timeCheck ) {
            try
            {
                $feed = \IPS\videos\Import::constructFromData( \IPS\Db::i()->select( '*', 'videos_rss_import', array( 'video_import_enabled=1 AND video_import_last_import<?', $timeCheck->getTimestamp() ), 'video_import_last_import ASC', 1 )->first() );
                $feed->run();
            }
            /* There's nothing more left to process */
            catch ( \UnderflowException $e )
            {
                /* Disable task if no enabled import feeds */
                if( !\IPS\Db::i()->select( 'count(video_import_id)', 'videos_rss_import', array( 'video_import_enabled=1' ), NULL, 1 )->first() )
                {
                    \IPS\Db::i()->update( 'core_tasks', array( 'enabled' => 0 ), array( '`key`=?', 'importVideos' ) );
                }

                /* No further processing needed */
                return FALSE;
            }
            /* Any other exception means an error which should be logged */
            catch ( \Exception $e )
            {
                /* If there is an error, we need to log it but the error should not prevent other feeds from importing */
                if ( isset( $feed ) AND ( $feed instanceof \IPS\videos\Import ) )
                {
                    $feed->last_import = time();
                    $feed->save();
                }
                throw new \IPS\Task\Exception( $this, $e->getMessage() );
            }

            /* Run again to see if there's anything left */
            return TRUE;
        });
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}